#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/debugfs.h>

#include <linux/ieee80211.h>
#include <net/mac80211.h>

#include "iwlwifi.h"
#include "iwl-priv.h"
#include "iwl-debug.h"
#include "iwl-io.h"

#define DEBUGFS_ADD_DIR(name, parent) do {                              \
	dbgfs->dir_##name = debugfs_create_dir(#name, parent);          \
	if (!(dbgfs->dir_##name))                                       \
	goto err;                                                       \
} while (0)

#define DEBUGFS_ADD_FILE(name, parent) do {                             \
	dbgfs->dbgfs_##parent##_files.file_##name =                     \
	debugfs_create_file(#name, 0644, dbgfs->dir_##parent, priv,     \
				&iwl_dbgfs_##name##_ops);               \
	if (!(dbgfs->dbgfs_##parent##_files.file_##name))               \
		goto err;                                               \
} while (0)

#define DEBUGFS_REMOVE(name)  do { 	\
	debugfs_remove(name); 		\
	name = NULL; 			\
} while (0);

#define DEBUGFS_READ_FUNC(name) 					\
static ssize_t iwl_dbgfs_##name##_read(struct file *file, 		\
					char __user *user_buf, 		\
					size_t count, loff_t *ppos);

#define DEBUGFS_WRITE_FUNC(name) 					\
static ssize_t iwl_dbgfs_##name##_write(struct file *file, 		\
					const char __user *user_buf, 	\
					size_t count, loff_t *ppos);

static int iwl_dbgfs_open_file_generic(struct inode *inode, struct file *file)
{
	file->private_data = inode->i_private;
	return 0;
}

#define DEBUGFS_READ_FILE_OPS(name) 	                                \
	DEBUGFS_READ_FUNC(name);                                        \
	static const struct file_operations iwl_dbgfs_##name##_ops = {  \
		.read = iwl_dbgfs_##name##_read,                        \
		.open = iwl_dbgfs_open_file_generic,                    \
};

#define DEBUGFS_READ_WRITE_FILE_OPS(name)  				\
	DEBUGFS_READ_FUNC(name);					\
	DEBUGFS_WRITE_FUNC(name);					\
static const struct file_operations iwl_dbgfs_##name##_ops = { 		\
	.write = iwl_dbgfs_##name##_write, 				\
	.read = iwl_dbgfs_##name##_read, 				\
	.open = iwl_dbgfs_open_file_generic, 				\
};

void iwl_dbgfs_txq_status(struct iwl_priv *priv,
					int txq_map, bool wake)
{
	if (wake)
		priv->dbgfs->txq_status |= txq_map;
	else
		priv->dbgfs->txq_status &= ~txq_map;
}

static ssize_t iwl_dbgfs_txq_read(struct file *file,
					char __user *user_buf,
					size_t count, loff_t *ppos) {

	struct iwl_priv *priv = (struct iwl_priv *)file->private_data;
	char buf[256];
	int pos = 0;
	int i;
	struct iwl_tx_queue *queue;
	u32 txq_status = priv->dbgfs->txq_status;

	pos += sprintf(buf+pos, "txq_status:\n");
	for (i = 0; i < IWL_MAX_NUM_QUEUES; i++)
		pos += sprintf(buf+pos, "%d\t", (txq_status >> i) & 0x1);

	pos += sprintf(buf+pos, "\nnum of tfds:\n");
	for (queue = &priv->txq[0]; queue != &(priv->txq[IWL_MAX_NUM_QUEUES]);
		queue++) {
		pos += sprintf(buf+pos, "%d\t",
			       (queue->q.n_bd - iwl_queue_space(&queue->q)-2));
	}

	pos += sprintf(buf+pos, "\n");

	return simple_read_from_buffer(user_buf, count, ppos, buf,
					pos);
}


static ssize_t iwl_dbgfs_tx_statistics_read(struct file *file,
						char __user *user_buf,
						size_t count, loff_t *ppos) {

	struct iwl_priv *priv = (struct iwl_priv *)file->private_data;
	char buf[32];
	int pos = 0;

	pos += sprintf(buf, "%u\n", priv->dbgfs->tx_statistics);
	return simple_read_from_buffer(user_buf, count, ppos, buf,
					pos);
}

static ssize_t iwl_dbgfs_rx_statistics_read(struct file *file,
						char __user *user_buf,
						size_t count, loff_t *ppos) {

	struct iwl_priv *priv = (struct iwl_priv *)file->private_data;
	char buf[256];
	int pos = 0;

	pos += sprintf(buf+pos, "managment: %u\n",
			priv->dbgfs->rx_statistics.managment_statistics);
	pos += sprintf(buf+pos, "data: %u\n",
			priv->dbgfs->rx_statistics.data_statistics);

	return simple_read_from_buffer(user_buf, count, ppos, buf,
					pos);
}

#define BYTE1_MASK 0x000000ff;
#define BYTE2_MASK 0x0000ffff;
#define BYTE3_MASK 0x00ffffff;
static ssize_t iwl_dbgfs_sram_read(struct file *file,
					char __user *user_buf,
					size_t count, loff_t *ppos)
{

	u32 val;
	char buf[1024];
	ssize_t ret;
	int i;
	int pos = 0;
	struct iwl_priv *priv = (struct iwl_priv *)file->private_data;

	printk(KERN_DEBUG "offset is: 0x%x\tlen is: 0x%x\n",
		priv->dbgfs->sram_offset, priv->dbgfs->sram_len);

	iwl_grab_nic_access(priv);
	for (i = priv->dbgfs->sram_len; i > 0; i -= 4) {
		val = iwl_read_targ_mem(priv, priv->dbgfs->sram_offset + \
						 priv->dbgfs->sram_len - i);
		if (i < 4) {
			switch (i) {
			case 1:
				val &= BYTE1_MASK;
				break;
			case 2:
				val &= BYTE2_MASK;
				break;
			case 3:
				val &= BYTE3_MASK;
				break;
			}
		}
		pos += sprintf(buf+pos, "0x%08x ", val);
	}
	pos += sprintf(buf+pos, "\n");
	iwl_release_nic_access(priv);

	ret = simple_read_from_buffer(user_buf, count, ppos, buf,
					pos);
	return ret;
}

static ssize_t iwl_dbgfs_sram_write(struct file *file,
					const char __user *user_buf,
					size_t count, loff_t *ppos)
{
	struct iwl_priv *priv = file->private_data;
	char buf[64];
	int buf_size;
	u32 offset, len;

	memset(buf, 0, sizeof(buf));
	buf_size = min(count, sizeof(buf) -  1);
	if (copy_from_user(buf, user_buf, buf_size))
		return -EFAULT;

	if (sscanf(buf, "%x,%x", &offset, &len) == 2) {
		priv->dbgfs->sram_offset = offset;
		priv->dbgfs->sram_len = len;
	} else {
		priv->dbgfs->sram_offset = 0;
		priv->dbgfs->sram_len = 0;
	}

	printk(KERN_DEBUG "offset is: 0x%x\tlen is: 0x%x\n",
	priv->dbgfs->sram_offset, priv->dbgfs->sram_len);
	return count;
}


static ssize_t iwl_dbgfs_stations_read(struct file *file,
					char __user *user_buf,
					size_t count, loff_t *ppos)
{
	int i, j;
	ssize_t ret;
	int pos = 0;
	struct iwl_priv *priv = (struct iwl_priv *)file->private_data;
	char *buf = (char *)kmalloc(sizeof(char) * 500 * (priv->num_stations),
					 GFP_KERNEL);
	struct iwl_station_entry *station;
	int max_sta = priv->hw_setting.max_stations;

	pos += sprintf(buf+pos, "num of stations: %d\n\n",
			priv->num_stations);
	for (station = priv->stations, i = 0; i < max_sta; station++, i++) {
		if (station->used == IWL_STA_USED) {
			pos += sprintf(buf+pos, "station %d:\ngeneral data:\n",
					i+1);
			pos += sprintf(buf+pos, "addr: " MAC_FMT "\n",
					MAC_ARG(station->sta.sta.addr));
			pos += sprintf(buf+pos, "id: %u\n",
					station->sta.sta.sta_id);
			pos += sprintf(buf+pos, "mode: %u\n",
					station->sta.mode);
			pos += sprintf(buf+pos, "flags: 0x%x\n",
					station->sta.station_flags_msk);
			pos += sprintf(buf+pos, "ps_status: %u\n",
					station->ps_status);

			pos += sprintf(buf+pos, "tid data:\n");

			pos += sprintf(buf+pos, "\t\tseq_num\ttfds_in_queue\t");
			pos += sprintf(buf+pos, "\ttxq_id\tframe_count\t");
			pos += sprintf(buf+pos, "wait_for_ba\t\tstart_idx\t");
			pos += sprintf(buf+pos, "bitmap\trate_n_flags\n");

			for (j = 0; j < MAX_TID_COUNT; j++) {
				pos += sprintf(buf+pos, "[%d]:\t\t%u\t\t%u\t\t",
						j, station->tid[j].seq_number,
						station->tid[j].tfds_in_queue);
				pos += sprintf(buf+pos, "%u\t\t%u\t\t%u\t\t",
						station->tid[j].agg.txq_id,
						station->tid[j].agg.frame_count,
						station->tid[j].agg.\
						wait_for_ba);
				pos += sprintf(buf+pos, "%u\t\t%llu\t\t%u\n",
						station->tid[j].agg.start_idx,
						station->tid[j].agg.bitmap,
						station->tid[j].agg.\
						rate_n_flags);
			}
			pos += sprintf(buf+pos, "\n");
		}
	}

	ret = simple_read_from_buffer(user_buf, count, ppos, buf,
					pos);
	kfree(buf);
	return ret;
}

DEBUGFS_READ_WRITE_FILE_OPS(sram);
DEBUGFS_READ_FILE_OPS(stations);
DEBUGFS_READ_FILE_OPS(rx_statistics);
DEBUGFS_READ_FILE_OPS(tx_statistics);
DEBUGFS_READ_FILE_OPS(txq);

/*
 * Create the debugfs files and directories
 *
 */
void iwl_dbgfs_register(struct iwl_priv *priv, const char *name)
{
	struct iwl_debugfs *dbgfs;

	dbgfs = kzalloc(sizeof(struct iwl_debugfs), GFP_KERNEL);
	if (!dbgfs)
		goto err;

	priv->dbgfs = dbgfs;
	dbgfs->name = name;
	dbgfs->dir_drv = debugfs_create_dir(name, NULL);
	if (!(dbgfs->dir_drv))
		goto err;
	DEBUGFS_ADD_DIR(data, dbgfs->dir_drv);
	DEBUGFS_ADD_FILE(sram, data);
	DEBUGFS_ADD_FILE(stations, data);
	DEBUGFS_ADD_FILE(rx_statistics, data);
	DEBUGFS_ADD_FILE(tx_statistics, data);
	DEBUGFS_ADD_FILE(txq, data);

	dbgfs->txq_status = 0xFFFF;

	return;

err:
		IWL_ERROR("Can't open the debugfs directory\n");
}
EXPORT_SYMBOL(iwl_dbgfs_register);

/**
 * Remove the debugfs files and directories
 *
 */
void iwl_dbgfs_unregister(struct iwl_priv *priv)
{
	if (!(priv->dbgfs))
		return;

	DEBUGFS_REMOVE(priv->dbgfs->dbgfs_data_files.file_txq);
	DEBUGFS_REMOVE(priv->dbgfs->dbgfs_data_files.file_rx_statistics);
	DEBUGFS_REMOVE(priv->dbgfs->dbgfs_data_files.file_tx_statistics);
	DEBUGFS_REMOVE(priv->dbgfs->dbgfs_data_files.file_sram);
	DEBUGFS_REMOVE(priv->dbgfs->dbgfs_data_files.file_stations);
	DEBUGFS_REMOVE(priv->dbgfs->dir_data);
	DEBUGFS_REMOVE(priv->dbgfs->dir_drv);
	kfree(priv->dbgfs);
	priv->dbgfs = NULL;
}
EXPORT_SYMBOL(iwl_dbgfs_unregister);

